'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _chaiSubset = require('chai-subset');

var _chaiSubset2 = _interopRequireDefault(_chaiSubset);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use(_chaiSubset2.default);

var fieldsSchema = {
  username: {
    value: 'john',
    disabled: true
  },
  password: 'doe',
  transport: {
    fields: ['feet', 'car', 'bicycle', 'moto', 'turbodesk'],
    value: ['car', 'turbodesk']
  },
  feet: {
    fields: ['left', 'right', 'both'],
    value: 'right'
  },
  car: {
    fields: ['toyota', 'mazda', 'ford'],
    value: ['toyota', 'ford']
  },
  color: {
    fields: {
      red: {},
      green: {},
      blue: { disabled: true }
    },
    value: 'red'
  },
  agree: false
};

var initializedFields = {
  username: {
    initialValue: 'john',
    value: 'john',
    disabled: true
  },
  password: {
    initialValue: 'doe',
    value: 'doe'
  },
  transport: {
    fields: {
      feet: { checked: false },
      car: { checked: true },
      bicycle: { checked: false },
      moto: { checked: false },
      turbodesk: { checked: true }
    },
    initialValue: ['car', 'turbodesk'],
    value: ['car', 'turbodesk']
  },
  feet: {
    fields: {
      left: { checked: false },
      right: { checked: true },
      both: { checked: false }
    },
    initialValue: ['right'],
    value: ['right']
  },
  car: {
    fields: {
      toyota: { checked: true },
      mazda: { checked: false },
      ford: { checked: true }
    },
    initialValue: ['toyota', 'ford'],
    value: ['toyota', 'ford']
  },
  color: {
    fields: {
      red: { checked: true },
      green: { checked: false },
      blue: { disabled: true, checked: false }
    },
    initialValue: ['red'],
    value: ['red']
  },
  agree: {
    initialValue: false,
    value: false,
    checked: false
  },
  nonFieldErrors: {
    errors: null
  }
};

describe('hoth-form utils', function () {
  describe('initFields', function () {
    it('accepts fields schema and returns initialized fields', function () {
      var initialized = (0, _utils.initFields)(fieldsSchema);
      (0, _chai.expect)(initialized).to.eql(initializedFields);
      (0, _chai.expect)(fieldsSchema).to.not.equal(initialized);
    });
  });

  describe('changeValue', function () {
    describe('accepts form fields and react element and returns form fields with updated value', function () {
      it('changes input value', function () {
        var changes = (0, _utils.changeValue)(initializedFields, { name: 'username', value: 'mark', type: 'input' });
        (0, _chai.expect)(changes).to.containSubset({
          username: {
            value: 'mark'
          }
        });
        (0, _chai.expect)(changes).to.not.equal(initializedFields);
      });

      it('changes radio value', function () {
        var changes = (0, _utils.changeValue)(initializedFields, { name: 'feet', value: 'both', type: 'radio' });
        (0, _chai.expect)(changes).to.containSubset({
          feet: {
            fields: {
              left: { checked: false },
              right: { checked: false },
              both: { checked: true }
            },
            value: ['both']
          }
        });
        (0, _chai.expect)(changes).to.not.equal(initializedFields);
      });

      it('changes single-checkbox value', function () {
        var changes = (0, _utils.changeValue)(initializedFields, { name: 'agree', value: 'false', type: 'checkbox', checked: true });
        (0, _chai.expect)(changes).to.containSubset({
          agree: {
            value: true,
            checked: true
          }
        });
        (0, _chai.expect)(changes).to.not.equal(initializedFields);
      });

      it('changes multiple-checkbox value', function () {
        var changes = (0, _utils.changeValue)(initializedFields, { name: 'transport', value: 'feet', checked: true, type: 'checkbox' });
        (0, _chai.expect)(changes).to.containSubset({
          transport: {
            fields: {
              feet: { checked: true },
              car: { checked: true },
              bicycle: { checked: false },
              moto: { checked: false },
              turbodesk: { checked: true }
            },
            value: ['feet', 'car', 'turbodesk']
          }
        });
        (0, _chai.expect)(changes).to.not.equal(initializedFields);
      });

      it('changes select value', function () {
        var changes = (0, _utils.changeValue)(initializedFields, { name: 'color', type: 'select', options: [{ value: 'red', selected: false }, { value: 'green', selected: true }, { value: 'blue', selected: false }] });
        (0, _chai.expect)(changes).to.containSubset({
          color: {
            fields: {
              red: { checked: false },
              green: { checked: true },
              blue: { checked: false }
            },
            value: ['green']
          }
        });
        (0, _chai.expect)(changes).to.not.equal(initializedFields);
      });

      it('changes multiple select value', function () {
        var changes = (0, _utils.changeValue)(initializedFields, { name: 'car', type: 'select-multiple', options: [{ value: 'toyota', selected: false }, { value: 'mazda', selected: true }, { value: 'ford', selected: true }] });
        (0, _chai.expect)(changes).to.containSubset({
          car: {
            fields: {
              toyota: { checked: false },
              mazda: { checked: true },
              ford: { checked: true }
            },
            value: ['mazda', 'ford']
          }
        });
        (0, _chai.expect)(changes).to.not.equal(initializedFields);
      });

      it('sets `dirty` flag if `value` differs from `initialValue`', function () {
        var changes = (0, _utils.changeValue)(initializedFields, { name: 'username', value: 'mark' });
        (0, _chai.expect)(changes.username.dirty).to.be.true;
        changes = (0, _utils.changeValue)(initializedFields, { name: 'username', value: 'john' });
        (0, _chai.expect)(changes.username.dirty).to.be.false;
      });
    });
  });

  describe('additionalChanges', function () {
    it('accepts form fields and callback function and returns updated form fields, but does not recalculate values', function () {
      var changes = (0, _utils.additionalChanges)(initializedFields, function (fields) {
        var result = {};
        result.username = { disabled: !fields.username.disabled };
        result.password = { disabled: fields.username.disabled };
        return result;
      });
      (0, _chai.expect)(changes).to.containSubset({
        username: {
          disabled: false
        },
        password: {
          disabled: true
        }
      });
      (0, _chai.expect)(changes).to.not.equal(initializedFields);
    });
  });

  describe('performValidation', function () {
    it('accepts form fields and callback function and returns from state with fields and errors flag', function () {
      var changes = (0, _utils.performValidation)(initializedFields, function (fields) {
        var result = {};
        if (fields.username.value === 'john') {
          result.username = 'This field is required';
        }
        result.nonFieldErrors = ['No', 'way'];
        return result;
      });
      (0, _chai.expect)(changes).to.containSubset({
        errors: true,
        fields: {
          username: {
            errors: ['This field is required']
          },
          nonFieldErrors: {
            errors: ['No', 'way']
          }
        }
      });
      (0, _chai.expect)(changes).to.not.equal(initializedFields);
    });
  });

  describe('resetFields', function () {
    it('resets all values', function () {
      var dirtyFields = _extends({}, initializedFields);
      dirtyFields.username.value = 'tom';
      dirtyFields.username.dirty = true;

      dirtyFields.agree.value = true;
      dirtyFields.agree.dirty = true;

      dirtyFields.color.value = ['green'];
      dirtyFields.color.dirty = true;

      var resettedFields = (0, _utils.resetFields)(dirtyFields);

      (0, _chai.expect)(resettedFields).to.containSubset({
        username: {
          value: 'john',
          dirty: false
        },
        agree: {
          value: false,
          dirty: false
        },
        color: {
          value: ['red'],
          dirty: false
        }
      });

      (0, _chai.expect)(dirtyFields).to.not.equal(resettedFields);
    });
  });
});